#include <LiquidCrystal_I2C.h>

#include <ros.h>
#include <std_msgs/Bool.h>
#include <std_msgs/Int32.h>

// Pin declaration
const int pullCordPin = D9;
const int switchPin = D3;
const int buttonPin = D10;
const int buttonLedPin = D11;

// Current team (either TEAM_YELLOW or TEAM_BLUE)
bool team;
bool buttonStatus;

// Last received estimated score
int estimatedScore = 0;

/**
 * Tirette (avec gestion du rebond)
 */

int pullCordState;            // the current reading from the input pin
int lastPullCordStatus = LOW; // the previous reading from the input pin

unsigned long lastDebounceTime = 0; // the last time the output pin was toggled
unsigned long debounceDelay = 50;   // the debounce time; increase if the output flickers

unsigned long lastMessageTime = 0;
unsigned int const MSG_SENDING_INTERVAL = 200;

ros::NodeHandle nh;

std_msgs::Bool teamMsg;
std_msgs::Bool buttonStatusMsg;
std_msgs::Bool pullCordStatusMsg;

ros::Publisher teamStatusPub("team", &teamMsg);
ros::Publisher buttonStatusPub("button_status", &buttonStatusMsg);
ros::Publisher pullCordStatusPub("pull_cord_status", &pullCordStatusMsg);

void estimatedScoreCallback(const std_msgs::Int32 &estimatedScore);
ros::Subscriber<std_msgs::Int32> estimatedScoreSub("estimated_score", &estimatedScoreCallback);

// LCD screen
LiquidCrystal_I2C lcd(0x27, 16, 2);

void refreshScreen()
{
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.print("Score=");
  lcd.setCursor(6, 0);
  lcd.print(estimatedScore);

  lcd.setCursor(0, 1);
  lcd.print(team ? "Team=JAUNE" : "Team=BLEU");
  lcd.setCursor(12, 1);
  lcd.print(pullCordState ? "GO" : "STOP");
  lcd.setCursor(13, 0);
  lcd.print(buttonStatus ? "ON" : "OFF");
}

void setup()
{
  pinMode(pullCordPin, INPUT_PULLUP);
  pinMode(switchPin, INPUT_PULLUP);
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(buttonLedPin, OUTPUT);

  digitalWrite(buttonLedPin, HIGH);

  lcd.init();
  lcd.backlight();

  team = digitalRead(switchPin);
  buttonStatus = digitalRead(buttonPin);

  nh.initNode();
  nh.advertise(teamStatusPub);
  nh.advertise(buttonStatusPub);
  nh.advertise(pullCordStatusPub);
  nh.subscribe(estimatedScoreSub);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Waiting ROS...");

  while (!nh.connected())
  {
    nh.spinOnce();
    delay(MSG_SENDING_INTERVAL);
  }

  refreshScreen();
}

// Appelé quand le score est mis à jour (message de score reçu sur l'Arduino depuis la raspi)
void estimatedScoreCallback(const std_msgs::Int32 &estimatedScoreMsg)
{
  estimatedScore = estimatedScoreMsg.data;
  refreshScreen();
}

void loop()
{

  if (!nh.connected())
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Waiting ROS...");

    while (!nh.connected())
    {
      nh.spinOnce();
      delay(MSG_SENDING_INTERVAL);
    }

    refreshScreen();
  }

  // DEBUT PARTIE TIRETTE

  int pullCordReading = digitalRead(pullCordPin);

  // If the switch changed, due to noise or pressing:
  if (pullCordReading != lastPullCordStatus)
  {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay)
  {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:

    // if the button state has changed:
    if (pullCordReading != pullCordState)
    {
      pullCordState = pullCordReading;
      refreshScreen();
    }
  }

  lastPullCordStatus = pullCordReading;

  // FIN PARTIE TIRETTE

  if (!digitalRead(buttonPin) != buttonStatus)
  {
    // Appui bouton
    buttonStatus = !digitalRead(buttonPin);
    refreshScreen();
  }

  if (digitalRead(switchPin) != team)
  {
    //Changement d'équipe
    team = digitalRead(switchPin);
    refreshScreen();
  }

  if (millis() - lastMessageTime >= MSG_SENDING_INTERVAL)
  {
    teamMsg.data = team;
    teamStatusPub.publish(&teamMsg);

    pullCordStatusMsg.data = pullCordState;
    pullCordStatusPub.publish(&pullCordStatusMsg);

    buttonStatusMsg.data = buttonStatus;
    buttonStatusPub.publish(&buttonStatusMsg);

    lastMessageTime = millis();
    nh.spinOnce();
  }
}